/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2007, Yann JOUANIN
 *
 * Yann JOUANIN yann.jouanin@intelunix.fr
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*! \file
 *
 * \brief Callback application
 *
 * \author Yann JOUANIN yann.jouanin@intelunix
 * 
 * This is CallBack application for Asterisk
 * \ingroup applications
 */

/*** MODULEINFO
	<defaultenabled>no</defaultenabled>
 ***/

#include "asterisk.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 1 $")

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "asterisk/file.h"
#include "asterisk/logger.h"
#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/module.h"
#include "asterisk/lock.h"
#include "asterisk/app.h"
#include "asterisk/astdb.h"

#define TIMEOUT 30
#define CALLBACK_TOUCH "5"

static char *app = "SetCallback";
static char *app2 = "GetCallback";
static char *synopsis = 
"Set Callback Application";
static char *synopsis2 =
"Get Callback Application";
static char *descrip = "Setting up a callback when busy.\n"
 " Syntax : SetCallback(callee,caller) \n";
static char *descrip2 ="Running callback if one have been set"
 " Callback(callee) \n";



static int app_setcallback_exec(struct ast_channel *chan, void *data)
{
	int res = 0,readres;
	char read_result[8], digit_file[32]="digits/", caller[256], callback_touch[]=CALLBACK_TOUCH;
	struct ast_module_user *u;
	struct ast_variable *var = NULL;
	struct ast_config *cfg = NULL;
	char *parse;
	
	AST_DECLARE_APP_ARGS(args,
		AST_APP_ARG(callee);
		AST_APP_ARG(caller);
	);
	
	
	cfg = ast_config_load("applications.conf");
	if (!cfg) {
		ast_log(LOG_WARNING,"Could not load applications.conf\n");
		return AST_MODULE_LOAD_DECLINE;
		}
	for (var = ast_variable_browse(cfg, "callback"); var; var = var->next) {
		if (!strcasecmp(var->name, "touch")) {
			ast_copy_string(callback_touch, var->value, sizeof(callback_touch));
		}
	}

			
	
	
	if (ast_strlen_zero(data)) {
		ast_log(LOG_WARNING, "%s requires two arguments (callee,caller) \n", app);
		return -1;
	}
	u = ast_module_user_add(chan);
	/* Do our thing here */
	/* We need to make a copy of the input string if we are going to modify it! */
	parse = ast_strdupa(data);
	AST_STANDARD_APP_ARGS(args, parse);
/*	if (args.argc == 2)
		ast_app_parse_options(app_opts, &flags, opts, args.options);
*/
	ast_stopstream(chan);
	res = ast_streamfile(chan, "callback-prompt", chan->language);
	if (!res) {
		res = ast_waitstream(chan, "");
		ast_stopstream(chan);
	} else {
		ast_log(LOG_WARNING, "ast_streamfile failed on %s for %s\n", chan->name, (char *)data);
	}
	strcat(digit_file,callback_touch);
	
	res = ast_streamfile(chan, "digit_file", chan->language);
	if (!res) {
		res = ast_waitstream(chan, "");
		ast_stopstream(chan);
	} else {
		ast_log(LOG_WARNING, "ast_streamfile failed on %s for %s\n", chan->name, (char *)data);
	}
	
	readres=ast_readstring  (chan,read_result,1,2*1000,5*1000,"");
	if (!readres && strcasecmp(read_result,callback_touch)==0) {
	
		ast_copy_string(caller, chan->tech->type, sizeof(caller));
		strcat(caller,"/");
		strcat(caller,args.caller);
		ast_db_put("Callback",args.callee, caller);
		ast_log(LOG_NOTICE, "Callback set from %s to %s\n", args.caller,args.callee);
	}
	ast_module_user_remove(u);
	return res;
}


static int app_getcallback_exec(struct ast_channel *chan, void *data)
{
	int reason = 0;
	int res = 0,get_result;
	char astdb_value[1024];
	char *number,*tech;
	
	struct ast_module_user *u;
	char *parse;
	
	AST_DECLARE_APP_ARGS(args,
		AST_APP_ARG(callee);
		
	);

	if (ast_strlen_zero(data)) {
		ast_log(LOG_WARNING, "%s requires an argument (callee) \n", app2);
		return -1;
	}

	u = ast_module_user_add(chan);


	parse = ast_strdupa(data);

	AST_STANDARD_APP_ARGS(args, parse);

	get_result=ast_db_get  ("Callback",args.callee,astdb_value,sizeof(astdb_value)); 
	if(!get_result) {
		number = astdb_value;
		tech = strsep(&number,"/");
		ast_log(LOG_NOTICE,"Tech:  %s, number %s \n",tech,number);
		ast_pbx_outgoing_exten(tech, AST_FORMAT_SLINEAR, number, TIMEOUT * 1000, "default", args.callee, 1, &reason, 1, number, "Callback", NULL, NULL, NULL);
		ast_log(LOG_NOTICE,"Calling back %s to %s\n",number,args.callee);
		ast_db_del  ("Callback",args.callee); 
	}
	else
		ast_log(LOG_NOTICE,"No callback set for %s\n",args.callee);
	
	ast_module_user_remove(u);
	return res;
}

static int unload_module(void)
{
	int res;
	res = ast_unregister_application(app);
	res |= ast_unregister_application(app2);
	
	return res;	
}

static int load_module(void)
{
	int res;
	res= ast_register_application(app, app_setcallback_exec, synopsis, descrip);
	res |= ast_register_application(app2, app_getcallback_exec, synopsis2, descrip2);
	return res;
}

AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Callback Application");
